//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  spimast.h                                                          *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  02/13/2003                                                        *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements an SPI master and slave message formats used *
//* by spimast.c, spicli.c, spislave.c, and spiserv.c applications.  This file is*
//* included by each of these applications so that they may communicate over the *
//* SPI bus using common message formats.                                        *                                                        *
//*                                                                              *
//*     This SPI slave controller can be used with a PC host or any SPI master,  *
//* in order to  give any PC the capability to serve as an SPI controller.       * 
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each SPI device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large          *
//* CRC and Checksum capabilities along with large message definitions for       *
//* various applications.                                                        *
//*                                                                              *
//* Usage:                                                                       *
//*	The following DC motor Controller command messages are available:            *
//*                                                                              * 
//* 	Cmm       - Calibrate motor speed for motor mm range (00..01). This      *
//*                 command is used to vary the motor speed and direction using  *
//*                 two potentiometers with the blades connected to AN0 and AN1. *
//*                 The true motor speed (RPM) is determined from the counts     *
//*                 received from the optical encoders (if used) and displayed.  *
//*                                                                              * 
//* 	Dmm,xxxx  - Send PWM duty command range xxxx (0000..FFFF) to motor       *
//*                 mm range (00..01). This command is used to vary              *
//*                 the motor speed and direction.                               *
//*                                                                              *
//* 	Emm       - Read DC motor encoders for position, speed and               *
//*                 acceleration and display the information for motor           *
//*                 mm range (00..01).                                           *
//*                                                                              *
//*     Fxx       - Select filter xx, where the filter ID                        *
//*                 range (00..FF) is in a 2 digit hex byte format,              *
//*                 where 00 => No Filter, 01 => Average Filter,                 *
//*                 02 => Kalman Filter2, 03 => Filter3,...                      *
//*                                                                              *
//* 	Gmm       - Get new motor commands from the current POT readings and     *
//*                 display them for the specific motor mm range (00..01).  This *
//*                 mode will remain in effect until the Hold command is issued. *
//*                                                                              *
//* 	Hmm       - Hold current POT reading and use it for the next PWM duty    *
//*                 command range (0000..FFFF) to motor                          *
//*                 mm range (00..01). This command is used to vary              * 
//*                 the motor speed and direction.                               *
//*                                                                              *
//*     Nxxxx     - Accept total number of samples, XXXX to be read from         *
//*                 selected Potentiometer, where xxxx is in 4 digit             *
//*                 hex byte format  range (00..FFFF)                            *
//*                                                                              *
//* 	Pmm,xx    - Send PWM period command range xx (00..FF) to motor           *
//*                 mm range (00..01). This command is used to vary the          *
//*                 PWM update frequency.                                        *
//*                                                                              *
//* 	Smm       - Read DC motor status including temperature, direction,       *
//*                 speed (RPM read from optical encoders and power state        * 
//*                 (ON/OFF) from motor mm range (00..01).                       *
//*                                                                              *
//* 	Tmm,xx    - Send trajectory profile xx range (00..FF) to motor mm        *
//*                 range (00..01).                                              *
//*                                                                              *
//* 	Vmm,xxxx  - Send velocity command range xxxx (0000..FFFF) to motor       *
//*                 mm range (00..01). This command is used to vary              * 
//*                 the motor speed and direction.                               *
//*                                                                              *
//* 	Zmm;       - Reset the DC motor Controller (brake the motor).  This      *
//*                 command can be used to implement an emergency stop           *
//*                 function.                                                    *
//*                                                                              *
//*                                                                              * 
//*   In order to use PWM2, the CCP2/RB3 pin fuse must be selected when          *
//*   programming the PIC.                                                       *                                                                                                       
//*                                                                              *
//* 	Example:                                                                 *
//*   		Send the following command string to the DC motor controller         *
//* 	to set PWM period to 0xFF using the string:                              *
//*       "P01,FF;"                                                              *
//* 	Where:                                                                   *
//*      	P - Is the motor command to change the PWM period.                   *
//*     	01 - Is the motor ID.                                                *
//*     	FF - Is the 2 character hex PWM period command.                      *
//*                                                                              *
//********************************************************************************
   
#ifndef SPIMAST_LOADED  /* prevent multiple loading */
#define SPIMAST_LOADED


#include "flags.h"                                  // Global flags used for MPSIM debugging 
#include "stdio.h"                                  // I/O support functions  
#include "network.h"                               	// Include support for networking
#include "spimsg.h"                                 // Include common messages
 // Port A

//#define SS			PORTFbits.RF7						// SPI Slave Co-processor Select/Status
//#define SS_DIR		TRISFbits.TRISF7					// SPI Slave Co-processor Select Direction/Status 
 
//#define SPI_CS		PORTAbits.RA5				   	    // SPI Slave Co-processor Select/status
//#define SPI_CS_DIR	TRISAbits.TRISA5			   	    // SPI Slave Co-Processor Select Direction
 
#define SPI_CS			PORTFbits.RF7						// SPI Slave Co-processor Select/Status
#define SPI_CS_DIR		TRISFbits.TRISF7					// SPI Slave Co-processor Select Direction/Status 

// Port B

#define TRACE0		PORTBbits.RB0						// Trace bit 0 for logic analyzer
#define TRACE0_DIR	TRISBbits.TRISB0

// Port C

#define SPI_SCL_DIR TRISCbits.TRISC3 					// SPI Clock Direction (Master)
#define SPI_SI_DIR TRISCbits.TRISC4 					// SPI Input Direction (Master)
#define SPI_SO_DIR TRISCbits.TRISC5 					// SPI Output Direction (Master)
#define SPI_SCL PORTCbits.RC3 							// SPI Clock Direction (Master)

// Port E

/*
#define SPI_CS		PORTEbits.RE2				   	    // SPI Slave Co-processor Select
#define SPI_CS_DIR	TRISEbits.TRISE2			   	    // SPI Slave Co-Processor Select Direction
*/
 

 // Declare Global SPI Master Buffer variables here

extern byte SPI_Rx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Receive Buffer
extern byte SPI_Tx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Transmit Buffer
extern byte SPI_Start_Buffer;                         		// Start pointer for ring buffer
extern byte SPI_End_Buffer;								// SPI End Buffer Index
extern byte SPI_Rx_Flag;                        		// Indicates that data was received into the ring buffer
extern word SPI_Timeout;                            		// SPI I/O Timeout counter
extern byte SPI_Timeout_Flag;                           		// SPI Timeout flag
extern word SPI_Number_Characters;						 	// SPI Number of characters in buffer
extern int SPI_Communications_Error;                    		// SPI I/O error flag
                                                 		// -1 = SPI Input Bufer overrun error
                                                 		// -2 = SPI Communications timeout error
                                                 		// -3 = SPI buffer empty error
/*
extern int SPI_Interrupt_Count;                   	// Number of MSSP interrupts
extern int SPI_Data_In;								// Data byte read from the SPI Master
extern int SPI_Data_Out;							// Data byte sent to the SPI Master
extern int SPI_Status;									// SPI Status returned from routines
*/
extern unsigned char TheCharacterRead;
extern unsigned char TheCharacterSent;

extern int MessageIndex;							// Message ID number

//**********************************************************************************
// Define messages for each type of command that is sent by the SPI Master and 
// recognized by the selected SPI Slave.
//**********************************************************************************

// SPI Master functions

//******************************************************************************
//* InitializeSPIMaster - Initialize the SPI Master hardware for SPI Clock
//* of Fosc/16, clock Iddle low (SPI Mode 0,0), and sample data after the
//* falling edge of SCK.   
//******************************************************************************
void InitializeSPIMaster(void);

/********************************************************************
*     Function Name:    ReadSPIByte                                 *
*     Return Value:     contents of SSPBUF register                 *
*     Parameters:       void                                        *
*     Description:      Read single byte from SPI bus (Master).     *
********************************************************************/
unsigned char ReadSPIByte( void );

/********************************************************************
*     Function Name:    WriteSPIByte                                *
*     Return Value:     Status byte for WCOL detection.             *
*     Parameters:       Single data byte for SPI bus.               *
*     Description:      This routine writes a single byte to the    * 
*                       SPI bus (Master).                           *
********************************************************************/
unsigned char WriteSPIByte( unsigned char data_out );

//**********************************************************************************
//* FlushSPIBuffers - Initialize contents of SPI Buffers. 
//**********************************************************************************
void FlushSPIBuffers(void);

//**********************************************************************************
//* CheckControllerState - Check the status of the the current controller to see if
//* the command has been executed correctly.  Check the following Controller state
//* variables: 
//*		ControllerState	{ NO_STATES=0, INITIALIZED=1, DONE=2, BUSY=3, ERROR=4 }
//* 	ControllerErrorCode  { NO_ERRORS=0, FAILURE=1, SUCCESS=2 }	 
//* 	ControllerDiagnosticCode { NO_DIAGNOSTICS=0, DIAGNOSTIC1=1, DIAGNOSTIC2=2 }    
//**********************************************************************************

byte CheckControllerState(void);

#endif

